@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/src/pcre/pcretest.c,v 1.8 2007/11/12 13:02:20 nm4 Exp $ */
+/* $Cambridge: exim/exim-src/src/pcre/pcretest.c,v 1.5 2006/12/19 14:38:38 ph10 Exp $ */
 
 /*************************************************
 *             PCRE testing program               *
@@ -38,10 +38,6 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
@@ -50,39 +46,18 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <locale.h>
 #include <errno.h>
 
-
-/* A number of things vary for Windows builds. Originally, pcretest opened its
-input and output without "b"; then I was told that "b" was needed in some
-environments, so it was added for release 5.0 to both the input and output. (It
-makes no difference on Unix-like systems.) Later I was told that it is wrong
-for the input on Windows. I've now abstracted the modes into two macros that
-are set here, to make it easier to fiddle with them, and removed "b" from the
-input mode under Windows. */
-
-#if defined(_WIN32) || defined(WIN32)
-#include <io.h>                /* For _setmode() */
-#include <fcntl.h>             /* For _O_BINARY */
-#define INPUT_MODE   "r"
-#define OUTPUT_MODE  "wb"
-
-#else
-#include <sys/time.h>          /* These two includes are needed */
-#include <sys/resource.h>      /* for setrlimit(). */
-#define INPUT_MODE   "rb"
-#define OUTPUT_MODE  "wb"
+#ifndef _WIN32
+#include <sys/time.h>
+#include <sys/resource.h>
 #endif
 
+#define PCRE_SPY        /* For Win32 build, import data, not export */
 
-/* We have to include pcre_internal.h because we need the internal info for
-displaying the results of pcre_study() and we also need to know about the
-internal macros, structures, and other internal data values; pcretest has
-"inside information" compared to a program that strictly follows the PCRE API.
+/* We include pcre_internal.h because we need the internal info for displaying
+the results of pcre_study() and we also need to know about the internal
+macros, structures, and other internal data values; pcretest has "inside
+information" compared to a program that strictly follows the PCRE API. */
 
-Although pcre_internal.h does itself include pcre.h, we explicitly include it
-here before pcre_internal.h so that the PCRE_EXP_xxx macros get set
-appropriately for an application, not for building PCRE. */
-
-#include "pcre.h"
 #include "pcre_internal.h"
 
 /* We need access to the data tables that PCRE uses. So as not to have to keep
@@ -96,25 +71,16 @@ symbols to prevent clashes. */
 #define _pcre_utf8_table4      utf8_table4
 #define _pcre_utt              utt
 #define _pcre_utt_size         utt_size
-#define _pcre_utt_names        utt_names
 #define _pcre_OP_lengths       OP_lengths
 
 #include "pcre_tables.c"
 
 /* We also need the pcre_printint() function for printing out compiled
 patterns. This function is in a separate file so that it can be included in
-pcre_compile.c when that module is compiled with debugging enabled.
-
-The definition of the macro PRINTABLE, which determines whether to print an
-output character as-is or as a hex value when showing compiled patterns, is
-contained in this file. We uses it here also, in cases when the locale has not
-been explicitly changed, so as to get consistent output from systems that
-differ in their output from isprint() even in the "C" locale. */
+pcre_compile.c when that module is compiled with debugging enabled. */
 
 #include "pcre_printint.src"
 
-#define PRINTHEX(c) (locale_set? isprint(c) : PRINTABLE(c))
-
 
 /* It is possible to compile this test program without including support for
 testing the POSIX interface, though this is not available via the standard
@@ -124,17 +90,10 @@ Makefile. */
 #include "pcreposix.h"
 #endif
 
-/* It is also possible, for the benefit of the version currently imported into
-Exim, to build pcretest without support for UTF8 (define NOUTF8), without the
-interface to the DFA matcher (NODFA), and without the doublecheck of the old
-"info" function (define NOINFOCHECK). In fact, we automatically cut out the
-UTF8 support if PCRE is built without it. */
-
-#ifndef SUPPORT_UTF8
-#ifndef NOUTF8
-#define NOUTF8
-#endif
-#endif
+/* It is also possible, for the benefit of the version imported into Exim, to
+build pcretest without support for UTF8 (define NOUTF8), without the interface
+to the DFA matcher (NODFA), and without the doublecheck of the old "info"
+function (define NOINFOCHECK). */
 
 
 /* Other parameters */
@@ -147,8 +106,6 @@ UTF8 support if PCRE is built without it. */
 #endif
 #endif
 
-/* This is the default loop count for timing. */
-
 #define LOOPREPEAT 500000
 
 /* Static variables */
@@ -159,9 +116,7 @@ static int callout_count;
 static int callout_extra;
 static int callout_fail_count;
 static int callout_fail_id;
-static int debug_lengths;
 static int first_callout;
-static int locale_set = 0;
 static int show_malloc;
 static int use_utf8;
 static size_t gotten_store;
@@ -205,7 +160,6 @@ uschar *here = start;
 for (;;)
   {
   int rlen = buffer_size - (here - buffer);
-
   if (rlen > 1000)
     {
     int dlen;
@@ -262,7 +216,7 @@ return NULL;  /* Control never gets here */
 
 /* We don't use strtoul() because SunOS4 doesn't have it. Rather than mess
 around with conditional compilation, just do the job by hand. It is only used
-for unpicking arguments, so just keep it simple.
+for unpicking the -o argument, so just keep it simple.
 
 Arguments:
   str           string to be converted
@@ -406,19 +360,16 @@ while (length-- > 0)
       {
       length -= rc - 1;
       p += rc;
-      if (PRINTHEX(c))
+      if (c < 256 && isprint(c))
         {
         if (f != NULL) fprintf(f, "%c", c);
         yield++;
         }
       else
         {
-        int n = 4;
-        if (f != NULL) fprintf(f, "\\x{%02x}", c);
-        yield += (n <= 0x000000ff)? 2 :
-                 (n <= 0x00000fff)? 3 :
-                 (n <= 0x0000ffff)? 4 :
-                 (n <= 0x000fffff)? 5 : 6;
+        int n;
+        if (f != NULL) fprintf(f, "\\x{%02x}%n", c, &n);
+        yield += n;
         }
       continue;
       }
@@ -427,8 +378,7 @@ while (length-- > 0)
 
    /* Not UTF-8, or malformed UTF-8  */
 
-  c = *p++;
-  if (PRINTHEX(c))
+  if (isprint(c = *(p++)))
     {
     if (f != NULL) fprintf(f, "%c", c);
     yield++;
@@ -666,39 +616,12 @@ return count;
 
 
 
-/*************************************************
-*         Case-independent strncmp() function    *
-*************************************************/
-
-/*
-Arguments:
-  s         first string
-  t         second string
-  n         number of characters to compare
-
-Returns:    < 0, = 0, or > 0, according to the comparison
-*/
-
-static int
-strncmpic(uschar *s, uschar *t, int n)
-{
-while (n--)
-  {
-  int c = tolower(*s++) - tolower(*t++);
-  if (c) return c;
-  }
-return 0;
-}
-
-
-
 /*************************************************
 *         Check newline indicator                *
 *************************************************/
 
 /* This is used both at compile and run-time to check for <xxx> escapes, where
-xxx is LF, CR, CRLF, ANYCRLF, or ANY. Print a message and return 0 if there is
-no match.
+xxx is LF, CR, or CRLF. Print a message and return 0 if there is no match.
 
 Arguments:
   p           points after the leading '<'
@@ -710,51 +633,15 @@ Returns:      appropriate PCRE_NEWLINE_xxx flags, or 0
 static int
 check_newline(uschar *p, FILE *f)
 {
-if (strncmpic(p, (uschar *)"cr>", 3) == 0) return PCRE_NEWLINE_CR;
-if (strncmpic(p, (uschar *)"lf>", 3) == 0) return PCRE_NEWLINE_LF;
-if (strncmpic(p, (uschar *)"crlf>", 5) == 0) return PCRE_NEWLINE_CRLF;
-if (strncmpic(p, (uschar *)"anycrlf>", 8) == 0) return PCRE_NEWLINE_ANYCRLF;
-if (strncmpic(p, (uschar *)"any>", 4) == 0) return PCRE_NEWLINE_ANY;
-if (strncmpic(p, (uschar *)"bsr_anycrlf>", 12) == 0) return PCRE_BSR_ANYCRLF;
-if (strncmpic(p, (uschar *)"bsr_unicode>", 12) == 0) return PCRE_BSR_UNICODE;
+if (strncmp((char *)p, "cr>", 3) == 0) return PCRE_NEWLINE_CR;
+if (strncmp((char *)p, "lf>", 3) == 0) return PCRE_NEWLINE_LF;
+if (strncmp((char *)p, "crlf>", 5) == 0) return PCRE_NEWLINE_CRLF;
 fprintf(f, "Unknown newline type at: <%s\n", p);
 return 0;
 }
 
 
 
-/*************************************************
-*             Usage function                     *
-*************************************************/
-
-static void
-usage(void)
-{
-printf("Usage:     pcretest [options] [<input> [<output>]]\n");
-printf("  -b       show compiled code (bytecode)\n");
-printf("  -C       show PCRE compile-time options and exit\n");
-printf("  -d       debug: show compiled code and information (-b and -i)\n");
-#if !defined NODFA
-printf("  -dfa     force DFA matching for all subjects\n");
-#endif
-printf("  -help    show usage information\n");
-printf("  -i       show information about compiled patterns\n"
-       "  -m       output memory used information\n"
-       "  -o <n>   set size of offsets vector to <n>\n");
-#if !defined NOPOSIX
-printf("  -p       use POSIX interface\n");
-#endif
-printf("  -q       quiet: do not output PCRE version number at start\n");
-printf("  -S <n>   set stack size to <n> megabytes\n");
-printf("  -s       output store (memory) used information\n"
-       "  -t       time compilation and execution\n");
-printf("  -t <n>   time compilation and execution, repeating <n> times\n");
-printf("  -tm      time execution (matching) only\n");
-printf("  -tm <n>  time execution (matching) only, repeating <n> times\n");
-}
-
-
-
 /*************************************************
 *                Main Program                    *
 *************************************************/
@@ -770,7 +657,6 @@ int options = 0;
 int study_options = 0;
 int op = 1;
 int timeit = 0;
-int timeitm = 0;
 int showinfo = 0;
 int showstore = 0;
 int quiet = 0;
@@ -802,18 +688,15 @@ buffer = (unsigned char *)malloc(buffer_size);
 dbuffer = (unsigned char *)malloc(buffer_size);
 pbuffer = (unsigned char *)malloc(buffer_size);
 
-/* The outfile variable is static so that new_malloc can use it. */
-
-outfile = stdout;
-
-/* The following  _setmode() stuff is some Windows magic that tells its runtime
-library to translate CRLF into a single LF character. At least, that's what
-I've been told: never having used Windows I take this all on trust. Originally
-it set 0x8000, but then I was advised that _O_BINARY was better. */
+/* The outfile variable is static so that new_malloc can use it. The _setmode()
+stuff is some magic that I don't understand, but which apparently does good
+things in Windows. It's related to line terminations.  */
 
 #if defined(_WIN32) || defined(WIN32)
-_setmode( _fileno( stdout ), _O_BINARY );
-#endif
+_setmode( _fileno( stdout ), 0x8000 );
+#endif  /* defined(_WIN32) || defined(WIN32) */
+
+outfile = stdout;
 
 /* Scan options */
 
@@ -823,8 +706,8 @@ while (argc > 1 && argv[op][0] == '-')
 
   if (strcmp(argv[op], "-s") == 0 || strcmp(argv[op], "-m") == 0)
     showstore = 1;
+  else if (strcmp(argv[op], "-t") == 0) timeit = 1;
   else if (strcmp(argv[op], "-q") == 0) quiet = 1;
-  else if (strcmp(argv[op], "-b") == 0) debug = 1;
   else if (strcmp(argv[op], "-i") == 0) showinfo = 1;
   else if (strcmp(argv[op], "-d") == 0) showinfo = debug = 1;
 #if !defined NODFA
@@ -837,25 +720,11 @@ while (argc > 1 && argv[op][0] == '-')
     op++;
     argc--;
     }
-  else if (strcmp(argv[op], "-t") == 0 || strcmp(argv[op], "-tm") == 0)
-    {
-    int both = argv[op][2] == 0;
-    int temp;
-    if (argc > 2 && (temp = get_value((unsigned char *)argv[op+1], &endptr),
-                     *endptr == 0))
-      {
-      timeitm = temp;
-      op++;
-      argc--;
-      }
-    else timeitm = LOOPREPEAT;
-    if (both) timeit = timeitm;
-    }
   else if (strcmp(argv[op], "-S") == 0 && argc > 2 &&
       ((stack_size = get_value((unsigned char *)argv[op+1], &endptr)),
         *endptr == 0))
     {
-#if defined(_WIN32) || defined(WIN32)
+#ifdef _WIN32
     printf("PCRE: -S not supported on this OS\n");
     exit(1);
 #else
@@ -887,12 +756,7 @@ while (argc > 1 && argv[op][0] == '-')
     printf("  %sUnicode properties support\n", rc? "" : "No ");
     (void)pcre_config(PCRE_CONFIG_NEWLINE, &rc);
     printf("  Newline sequence is %s\n", (rc == '\r')? "CR" :
-      (rc == '\n')? "LF" : (rc == ('\r'<<8 | '\n'))? "CRLF" :
-      (rc == -2)? "ANYCRLF" :
-      (rc == -1)? "ANY" : "???");
-    (void)pcre_config(PCRE_CONFIG_BSR, &rc);
-    printf("  \\R matches %s\n", rc? "CR, LF, or CRLF only" :
-                                     "all Unicode newlines");
+      (rc == '\n')? "LF" : "CRLF");
     (void)pcre_config(PCRE_CONFIG_LINK_SIZE, &rc);
     printf("  Internal link size = %d\n", rc);
     (void)pcre_config(PCRE_CONFIG_POSIX_MALLOC_THRESHOLD, &rc);
@@ -903,18 +767,26 @@ while (argc > 1 && argv[op][0] == '-')
     printf("  Default recursion depth limit = %d\n", rc);
     (void)pcre_config(PCRE_CONFIG_STACKRECURSE, &rc);
     printf("  Match recursion uses %s\n", rc? "stack" : "heap");
-    goto EXIT;
-    }
-  else if (strcmp(argv[op], "-help") == 0 ||
-           strcmp(argv[op], "--help") == 0)
-    {
-    usage();
-    goto EXIT;
+    exit(0);
     }
   else
     {
     printf("** Unknown or malformed option %s\n", argv[op]);
-    usage();
+    printf("Usage:   pcretest [options] [<input> [<output>]]\n");
+    printf("  -C     show PCRE compile-time options and exit\n");
+    printf("  -d     debug: show compiled code; implies -i\n");
+#if !defined NODFA
+    printf("  -dfa   force DFA matching for all subjects\n");
+#endif
+    printf("  -i     show information about compiled pattern\n"
+           "  -m     output memory used information\n"
+           "  -o <n> set size of offsets vector to <n>\n");
+#if !defined NOPOSIX
+    printf("  -p     use POSIX interface\n");
+#endif
+    printf("  -S <n> set stack size to <n> megabytes\n");
+    printf("  -s     output store (memory) used information\n"
+           "  -t     time compilation and execution\n");
     yield = 1;
     goto EXIT;
     }
@@ -929,7 +801,7 @@ offsets = (int *)malloc(size_offsets_max * sizeof(int));
 if (offsets == NULL)
   {
   printf("** Failed to get %d bytes of memory for offsets vector\n",
-    (int)(size_offsets_max * sizeof(int)));
+    size_offsets_max * sizeof(int));
   yield = 1;
   goto EXIT;
   }
@@ -938,7 +810,7 @@ if (offsets == NULL)
 
 if (argc > 1)
   {
-  infile = fopen(argv[op], INPUT_MODE);
+  infile = fopen(argv[op], "rb");
   if (infile == NULL)
     {
     printf("** Failed to open %s\n", argv[op]);
@@ -949,7 +821,7 @@ if (argc > 1)
 
 if (argc > 2)
   {
-  outfile = fopen(argv[op+1], OUTPUT_MODE);
+  outfile = fopen(argv[op+1], "wb");
   if (outfile == NULL)
     {
     printf("** Failed to open %s\n", argv[op+1]);
@@ -994,10 +866,9 @@ while (!done)
   int do_showinfo = showinfo;
   int do_showrest = 0;
   int do_flip = 0;
-  int erroroffset, len, delimiter, poffset;
+  int erroroffset, len, delimiter;
 
   use_utf8 = 0;
-  debug_lengths = 1;
 
   if (infile == stdin) printf("  re> ");
   if (extend_inputline(infile, buffer) == NULL) break;
@@ -1105,7 +976,6 @@ while (!done)
     }
 
   pp = p;
-  poffset = p - buffer;
 
   for(;;)
     {
@@ -1126,11 +996,6 @@ while (!done)
     if (infile != stdin) fprintf(outfile, "%s", (char *)pp);
     }
 
-  /* The buffer may have moved while being extended; reset the start of data
-  pointer to the correct relative point in the buffer. */
-
-  p = buffer + poffset;
-
   /* If the first character after the delimiter is backslash, make
   the pattern end with backslash. This is purely to provide a way
   of testing for the error message when a pattern ends with backslash. */
@@ -1162,7 +1027,6 @@ while (!done)
 
       case '+': do_showrest = 1; break;
       case 'A': options |= PCRE_ANCHORED; break;
-      case 'B': do_debug = 1; break;
       case 'C': options |= PCRE_AUTO_CALLOUT; break;
       case 'D': do_debug = do_showinfo = 1; break;
       case 'E': options |= PCRE_DOLLAR_ENDONLY; break;
@@ -1180,22 +1044,19 @@ while (!done)
       case 'S': do_study = 1; break;
       case 'U': options |= PCRE_UNGREEDY; break;
       case 'X': options |= PCRE_EXTRA; break;
-      case 'Z': debug_lengths = 0; break;
       case '8': options |= PCRE_UTF8; use_utf8 = 1; break;
       case '?': options |= PCRE_NO_UTF8_CHECK; break;
 
       case 'L':
       ppp = pp;
-      /* The '\r' test here is so that it works on Windows. */
-      /* The '0' test is just in case this is an unterminated line. */
-      while (*ppp != 0 && *ppp != '\n' && *ppp != '\r' && *ppp != ' ') ppp++;
+      /* The '\r' test here is so that it works on Windows */
+      while (*ppp != '\n' && *ppp != '\r' && *ppp != ' ') ppp++;
       *ppp = 0;
       if (setlocale(LC_CTYPE, (const char *)pp) == NULL)
         {
         fprintf(outfile, "** Failed to set locale \"%s\"\n", pp);
         goto SKIP_DATA;
         }
-      locale_set = 1;
       tables = pcre_maketables();
       pp = ppp;
       break;
@@ -1262,19 +1123,19 @@ while (!done)
 #endif  /* !defined NOPOSIX */
 
     {
-    if (timeit > 0)
+    if (timeit)
       {
       register int i;
       clock_t time_taken;
       clock_t start_time = clock();
-      for (i = 0; i < timeit; i++)
+      for (i = 0; i < LOOPREPEAT; i++)
         {
         re = pcre_compile((char *)p, options, &error, &erroroffset, tables);
         if (re != NULL) free(re);
         }
       time_taken = clock() - start_time;
-      fprintf(outfile, "Compile time %.4f milliseconds\n",
-        (((double)time_taken * 1000.0) / (double)timeit) /
+      fprintf(outfile, "Compile time %.3f milliseconds\n",
+        (((double)time_taken * 1000.0) / (double)LOOPREPEAT) /
           (double)CLOCKS_PER_SEC);
       }
 
@@ -1326,17 +1187,17 @@ while (!done)
 
     if (do_study)
       {
-      if (timeit > 0)
+      if (timeit)
         {
         register int i;
         clock_t time_taken;
         clock_t start_time = clock();
-        for (i = 0; i < timeit; i++)
+        for (i = 0; i < LOOPREPEAT; i++)
           extra = pcre_study(re, study_options, &error);
         time_taken = clock() - start_time;
         if (extra != NULL) free(extra);
-        fprintf(outfile, "  Study time %.4f milliseconds\n",
-          (((double)time_taken * 1000.0) / (double)timeit) /
+        fprintf(outfile, "  Study time %.3f milliseconds\n",
+          (((double)time_taken * 1000.0) / (double)LOOPREPEAT) /
             (double)CLOCKS_PER_SEC);
         }
       extra = pcre_study(re, study_options, &error);
@@ -1354,25 +1215,18 @@ while (!done)
     if (do_flip)
       {
       real_pcre *rre = (real_pcre *)re;
-      rre->magic_number =
-        byteflip(rre->magic_number, sizeof(rre->magic_number));
+      rre->magic_number = byteflip(rre->magic_number, sizeof(rre->magic_number));
       rre->size = byteflip(rre->size, sizeof(rre->size));
       rre->options = byteflip(rre->options, sizeof(rre->options));
-      rre->flags = (pcre_uint16)byteflip(rre->flags, sizeof(rre->flags));
-      rre->top_bracket =
-        (pcre_uint16)byteflip(rre->top_bracket, sizeof(rre->top_bracket));
-      rre->top_backref =
-        (pcre_uint16)byteflip(rre->top_backref, sizeof(rre->top_backref));
-      rre->first_byte =
-        (pcre_uint16)byteflip(rre->first_byte, sizeof(rre->first_byte));
-      rre->req_byte =
-        (pcre_uint16)byteflip(rre->req_byte, sizeof(rre->req_byte));
-      rre->name_table_offset = (pcre_uint16)byteflip(rre->name_table_offset,
+      rre->top_bracket = byteflip(rre->top_bracket, sizeof(rre->top_bracket));
+      rre->top_backref = byteflip(rre->top_backref, sizeof(rre->top_backref));
+      rre->first_byte = byteflip(rre->first_byte, sizeof(rre->first_byte));
+      rre->req_byte = byteflip(rre->req_byte, sizeof(rre->req_byte));
+      rre->name_table_offset = byteflip(rre->name_table_offset,
         sizeof(rre->name_table_offset));
-      rre->name_entry_size = (pcre_uint16)byteflip(rre->name_entry_size,
+      rre->name_entry_size = byteflip(rre->name_entry_size,
         sizeof(rre->name_entry_size));
-      rre->name_count = (pcre_uint16)byteflip(rre->name_count,
-        sizeof(rre->name_count));
+      rre->name_count = byteflip(rre->name_count, sizeof(rre->name_count));
 
       if (extra != NULL)
         {
@@ -1386,23 +1240,22 @@ while (!done)
 
     SHOW_INFO:
 
-    if (do_debug)
-      {
-      fprintf(outfile, "------------------------------------------------------------------\n");
-      pcre_printint(re, outfile, debug_lengths);
-      }
-
     if (do_showinfo)
       {
       unsigned long int get_options, all_options;
 #if !defined NOINFOCHECK
       int old_first_char, old_options, old_count;
 #endif
-      int count, backrefmax, first_char, need_char, okpartial, jchanged,
-        hascrorlf;
+      int count, backrefmax, first_char, need_char;
       int nameentrysize, namecount;
       const uschar *nametable;
 
+      if (do_debug)
+        {
+        fprintf(outfile, "------------------------------------------------------------------\n");
+        pcre_printint(re, outfile);
+        }
+
       new_info(re, NULL, PCRE_INFO_OPTIONS, &get_options);
       new_info(re, NULL, PCRE_INFO_SIZE, &size);
       new_info(re, NULL, PCRE_INFO_CAPTURECOUNT, &count);
@@ -1412,9 +1265,6 @@ while (!done)
       new_info(re, NULL, PCRE_INFO_NAMEENTRYSIZE, &nameentrysize);
       new_info(re, NULL, PCRE_INFO_NAMECOUNT, &namecount);
       new_info(re, NULL, PCRE_INFO_NAMETABLE, (void *)&nametable);
-      new_info(re, NULL, PCRE_INFO_OKPARTIAL, &okpartial);
-      new_info(re, NULL, PCRE_INFO_JCHANGED, &jchanged);
-      new_info(re, NULL, PCRE_INFO_HASCRORLF, &hascrorlf);
 
 #if !defined NOINFOCHECK
       old_count = pcre_info(re, &old_options, &old_first_char);
@@ -1456,22 +1306,26 @@ while (!done)
           }
         }
 
-      if (!okpartial) fprintf(outfile, "Partial matching not supported\n");
-      if (hascrorlf) fprintf(outfile, "Contains explicit CR or LF match\n");
+      /* The NOPARTIAL bit is a private bit in the options, so we have
+      to fish it out via out back door */
 
       all_options = ((real_pcre *)re)->options;
-      if (do_flip) all_options = byteflip(all_options, sizeof(all_options));
+      if (do_flip)
+        {
+        all_options = byteflip(all_options, sizeof(all_options));
+         }
+
+      if ((all_options & PCRE_NOPARTIAL) != 0)
+        fprintf(outfile, "Partial matching not supported\n");
 
       if (get_options == 0) fprintf(outfile, "No options\n");
-        else fprintf(outfile, "Options:%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s\n",
+        else fprintf(outfile, "Options:%s%s%s%s%s%s%s%s%s%s%s%s%s\n",
           ((get_options & PCRE_ANCHORED) != 0)? " anchored" : "",
           ((get_options & PCRE_CASELESS) != 0)? " caseless" : "",
           ((get_options & PCRE_EXTENDED) != 0)? " extended" : "",
           ((get_options & PCRE_MULTILINE) != 0)? " multiline" : "",
           ((get_options & PCRE_FIRSTLINE) != 0)? " firstline" : "",
           ((get_options & PCRE_DOTALL) != 0)? " dotall" : "",
-          ((get_options & PCRE_BSR_ANYCRLF) != 0)? " bsr_anycrlf" : "",
-          ((get_options & PCRE_BSR_UNICODE) != 0)? " bsr_unicode" : "",
           ((get_options & PCRE_DOLLAR_ENDONLY) != 0)? " dollar_endonly" : "",
           ((get_options & PCRE_EXTRA) != 0)? " extra" : "",
           ((get_options & PCRE_UNGREEDY) != 0)? " ungreedy" : "",
@@ -1480,9 +1334,7 @@ while (!done)
           ((get_options & PCRE_NO_UTF8_CHECK) != 0)? " no_utf8_check" : "",
           ((get_options & PCRE_DUPNAMES) != 0)? " dupnames" : "");
 
-      if (jchanged) fprintf(outfile, "Duplicate name status changes\n");
-
-      switch (get_options & PCRE_NEWLINE_BITS)
+      switch (get_options & PCRE_NEWLINE_CRLF)
         {
         case PCRE_NEWLINE_CR:
         fprintf(outfile, "Forced newline sequence: CR\n");
@@ -1496,14 +1348,6 @@ while (!done)
         fprintf(outfile, "Forced newline sequence: CRLF\n");
         break;
 
-        case PCRE_NEWLINE_ANYCRLF:
-        fprintf(outfile, "Forced newline sequence: ANYCRLF\n");
-        break;
-
-        case PCRE_NEWLINE_ANY:
-        fprintf(outfile, "Forced newline sequence: ANY\n");
-        break;
-
         default:
         break;
         }
@@ -1521,7 +1365,7 @@ while (!done)
         int ch = first_char & 255;
         const char *caseless = ((first_char & REQ_CASELESS) == 0)?
           "" : " (caseless)";
-        if (PRINTHEX(ch))
+        if (isprint(ch))
           fprintf(outfile, "First char = \'%c\'%s\n", ch, caseless);
         else
           fprintf(outfile, "First char = %d%s\n", ch, caseless);
@@ -1536,7 +1380,7 @@ while (!done)
         int ch = need_char & 255;
         const char *caseless = ((need_char & REQ_CASELESS) == 0)?
           "" : " (caseless)";
-        if (PRINTHEX(ch))
+        if (isprint(ch))
           fprintf(outfile, "Need char = \'%c\'%s\n", ch, caseless);
         else
           fprintf(outfile, "Need char = %d%s\n", ch, caseless);
@@ -1572,7 +1416,7 @@ while (!done)
                   fprintf(outfile, "\n  ");
                   c = 2;
                   }
-                if (PRINTHEX(i) && i != ' ')
+                if (isprint(i) && i != ' ')
                   {
                   fprintf(outfile, "%c ", i);
                   c += 2;
@@ -1604,15 +1448,15 @@ while (!done)
       else
         {
         uschar sbuf[8];
-        sbuf[0] = (uschar)((true_size >> 24) & 255);
-        sbuf[1] = (uschar)((true_size >> 16) & 255);
-        sbuf[2] = (uschar)((true_size >>  8) & 255);
-        sbuf[3] = (uschar)((true_size) & 255);
+        sbuf[0] = (true_size >> 24)  & 255;
+        sbuf[1] = (true_size >> 16)  & 255;
+        sbuf[2] = (true_size >>  8)  & 255;
+        sbuf[3] = (true_size)  & 255;
 
-        sbuf[4] = (uschar)((true_study_size >> 24) & 255);
-        sbuf[5] = (uschar)((true_study_size >> 16) & 255);
-        sbuf[6] = (uschar)((true_study_size >>  8) & 255);
-        sbuf[7] = (uschar)((true_study_size) & 255);
+        sbuf[4] = (true_study_size >> 24)  & 255;
+        sbuf[5] = (true_study_size >> 16)  & 255;
+        sbuf[6] = (true_study_size >>  8)  & 255;
+        sbuf[7] = (true_study_size)  & 255;
 
         if (fwrite(sbuf, 1, 8, f) < 8 ||
             fwrite(re, 1, true_size, f) < true_size)
@@ -1631,7 +1475,6 @@ while (!done)
                 strerror(errno));
               }
             else fprintf(outfile, "Study data written to %s\n", to_file);
-
             }
           }
         fclose(f);
@@ -1649,7 +1492,7 @@ while (!done)
   for (;;)
     {
     uschar *q;
-    uschar *bptr;
+    uschar *bptr = dbuffer;
     int *use_offsets = offsets;
     int use_size_offsets = size_offsets;
     int callout_data = 0;
@@ -1705,7 +1548,7 @@ while (!done)
     p = buffer;
     while (isspace(*p)) p++;
 
-    bptr = q = dbuffer;
+    q = dbuffer;
     while ((c = *p++) != 0)
       {
       int i = 0;
@@ -1900,7 +1743,7 @@ while (!done)
           if (offsets == NULL)
             {
             printf("** Failed to get %d bytes of memory for offsets vector\n",
-              (int)(size_offsets_max * sizeof(int)));
+              size_offsets_max * sizeof(int));
             yield = 1;
             goto EXIT;
             }
@@ -2030,7 +1873,7 @@ while (!done)
 
     for (;; gmatched++)    /* Loop for /g or /G */
       {
-      if (timeitm > 0)
+      if (timeit)
         {
         register int i;
         clock_t time_taken;
@@ -2040,7 +1883,7 @@ while (!done)
         if (all_use_dfa || use_dfa)
           {
           int workspace[1000];
-          for (i = 0; i < timeitm; i++)
+          for (i = 0; i < LOOPREPEAT; i++)
             count = pcre_dfa_exec(re, NULL, (char *)bptr, len, start_offset,
               options | g_notempty, use_offsets, use_size_offsets, workspace,
               sizeof(workspace)/sizeof(int));
@@ -2048,13 +1891,13 @@ while (!done)
         else
 #endif
 
-        for (i = 0; i < timeitm; i++)
+        for (i = 0; i < LOOPREPEAT; i++)
           count = pcre_exec(re, extra, (char *)bptr, len,
             start_offset, options | g_notempty, use_offsets, use_size_offsets);
 
         time_taken = clock() - start_time;
-        fprintf(outfile, "Execute time %.4f milliseconds\n",
-          (((double)time_taken * 1000.0) / (double)timeitm) /
+        fprintf(outfile, "Execute time %.3f milliseconds\n",
+          (((double)time_taken * 1000.0) / (double)LOOPREPEAT) /
             (double)CLOCKS_PER_SEC);
         }
 
@@ -2130,28 +1973,7 @@ while (!done)
 
       if (count >= 0)
         {
-        int i, maxcount;
-
-#if !defined NODFA
-        if (all_use_dfa || use_dfa) maxcount = use_size_offsets/2; else
-#endif
-          maxcount = use_size_offsets/3;
-
-        /* This is a check against a lunatic return value. */
-
-        if (count > maxcount)
-          {
-          fprintf(outfile,
-            "** PCRE error: returned count %d is too big for offset size %d\n",
-            count, use_size_offsets);
-          count = use_size_offsets/3;
-          if (do_g || do_G)
-            {
-            fprintf(outfile, "** /%c loop abandoned\n", do_g? 'g' : 'G');
-            do_g = do_G = FALSE;        /* Break g/G loop */
-            }
-          }
-
+        int i;
         for (i = 0; i < count * 2; i += 2)
           {
           if (use_offsets[i] < 0)
@@ -2269,44 +2091,19 @@ while (!done)
         }
 
       /* Failed to match. If this is a /g or /G loop and we previously set
-      g_notempty after a null match, this is not necessarily the end. We want
-      to advance the start offset, and continue. We won't be at the end of the
-      string - that was checked before setting g_notempty.
-
-      Complication arises in the case when the newline option is "any" or
-      "anycrlf". If the previous match was at the end of a line terminated by
-      CRLF, an advance of one character just passes the \r, whereas we should
-      prefer the longer newline sequence, as does the code in pcre_exec().
-      Fudge the offset value to achieve this.
-
-      Otherwise, in the case of UTF-8 matching, the advance must be one
-      character, not one byte. */
+      g_notempty after a null match, this is not necessarily the end.
+      We want to advance the start offset, and continue. In the case of UTF-8
+      matching, the advance must be one character, not one byte. Fudge the
+      offset values to achieve this. We won't be at the end of the string -
+      that was checked before setting g_notempty. */
 
       else
         {
         if (g_notempty != 0)
           {
           int onechar = 1;
-          unsigned int obits = ((real_pcre *)re)->options;
           use_offsets[0] = start_offset;
-          if ((obits & PCRE_NEWLINE_BITS) == 0)
-            {
-            int d;
-            (void)pcre_config(PCRE_CONFIG_NEWLINE, &d);
-            obits = (d == '\r')? PCRE_NEWLINE_CR :
-                    (d == '\n')? PCRE_NEWLINE_LF :
-                    (d == ('\r'<<8 | '\n'))? PCRE_NEWLINE_CRLF :
-                    (d == -2)? PCRE_NEWLINE_ANYCRLF :
-                    (d == -1)? PCRE_NEWLINE_ANY : 0;
-            }
-          if (((obits & PCRE_NEWLINE_BITS) == PCRE_NEWLINE_ANY ||
-               (obits & PCRE_NEWLINE_BITS) == PCRE_NEWLINE_ANYCRLF)
-              &&
-              start_offset < len - 1 &&
-              bptr[start_offset] == '\r' &&
-              bptr[start_offset+1] == '\n')
-            onechar++;
-          else if (use_utf8)
+          if (use_utf8)
             {
             while (start_offset + onechar < len)
               {
@@ -2341,7 +2138,6 @@ while (!done)
       character. */
 
       g_notempty = 0;
-
       if (use_offsets[0] == use_offsets[1])
         {
         if (use_offsets[0] == len) break;
@@ -2376,7 +2172,6 @@ while (!done)
     {
     new_free((void *)tables);
     setlocale(LC_CTYPE, "C");
-    locale_set = 0;
     }
   }
 
